from awesomeNations.exceptions import NationNotFound
from awesomeNations.sharedMethods import request, format_text
from awesomeNations.configuration import DEFAULT_PARSER
from awesomeNations.seleniumScrapper import get_dynamic_source, script_runner
from bs4 import BeautifulSoup as bs
from typing import Iterator
from concurrent.futures import ThreadPoolExecutor

def nationBubbles(top, bottom) -> dict:
        bubble_keys = [format_text(title.get_text()) for title in top]
        bubble_values = [key.get_text() for key in bottom]
        bubbles = {}

        for i in range(len(bubble_keys)):
            bubbles.update({bubble_keys[i]: bubble_values[i]})

        return bubbles

def nationSummary(nationsummary) -> dict:
    government = f'{nationsummary[0].get_text()}\n{nationsummary[1].get_text()}'
    economy = nationsummary[2].get_text()
    more = nationsummary[3].get_text()
    description = dict(government=government, economy=economy, more=more)

    return description

def summaryBox(box) -> dict:
    items = [item.get_text() for item in box]
    population = items[items.index('Population') + 1] if 'Population' in items else '...'
    capital = items[items.index('Capital') + 1] if 'Capital' in items else '...'
    leader = items[items.index('Leader') + 1] if 'Leader' in items else '...'
    faith = items[items.index('Faith') + 1] if 'Faith' in items else '...'
    currency = items[items.index('Currency') + 1] if 'Currency' in items else '...'
    animal = items[items.index('Animal') + 1] if 'Animal' in items else '...'

    values = dict(population=population, capital=capital, leader=leader, faith=faith, currency=currency, animal=animal)
    return values

def census_url_generator(nation_name: str, id: tuple | list) -> Iterator:
      for id in id:
        yield f'https://www.nationstates.net/nation={nation_name}/detail=trend/censusid={id}'

def scrape_census(url: str) -> dict:
    try:
        response = request(url=url)
        soup: bs = response

        title = soup.find('h2').get_text()
        value = soup.find('div', class_='censusscoreboxtop').get_text().replace(' ', '')
        bubble_top_line = soup.find_all('div', class_='newmainlinebubbletop')
        bubble_bottom_line = soup.find_all('div', class_='newmainlinebubblebottom')
        bubbles = nationBubbles(bubble_top_line, bubble_bottom_line)
        return {'title': title, 'value': value, 'bubbles': bubbles}
    except:
        return None

def check_if_nation_exists(nation_name: str) -> None:
    url: str = f'https://www.nationstates.net/nation={nation_name}'
    response: dict = request(parser=DEFAULT_PARSER, url=url)
    soup: bs = response
    error_p = soup.find('p', class_="error")

    if error_p:
        raise NationNotFound(nation_name)

class N:
    def __init__(self, nation_name: str) -> None:
        self.nation_name = nation_name

    def exists(self) -> bool:
        nation_name: str = self.nation_name
        url: str = f'https://www.nationstates.net/nation={nation_name}'
        response = request(parser='html.parser', url=url)
        soup: bs = response
        error_p = soup.find('p', class_="error")

        if error_p:
            return False
        else:
            return True

    def overview(self) -> dict:
        nation_name: str = self.nation_name
        formatted_name = format_text(nation_name)
        check_if_nation_exists(formatted_name)

        url: str = f'https://www.nationstates.net/nation={formatted_name}'
        response = request(parser='lxml', url=url)
        soup: bs = response

        flag_source = soup.find('div', class_='newflagbox').find('img').extract()
        if flag_source:
            if flag_source.has_attr('src'):
                flag = f'www.nationstates.net{flag_source.attrs['src']}'
            elif flag_source.has_attr('data-cfsrc'):
                flag = f'www.nationstates.net{flag_source.attrs['data-cfsrc']}'

        short_name = soup.find('div', class_='newtitlename').get_text().replace('\n', '')
        long_name = f'{soup.find('div', class_='newtitlepretitle').get_text()} {short_name}'.replace('\n', '')
        wa_category = soup.find('div', class_='newtitlecategory').get_text()
        motto = soup.find('span', class_='slogan').get_text()

        bubble_top_line = soup.find_all('div', class_='newmainlinebubbletop')
        bubble_bottom_line = soup.find_all('div', class_='newmainlinebubblebottom')
        bubbles = nationBubbles(bubble_top_line, bubble_bottom_line)

        nationsummary = soup.find('div', class_='nationsummary').find_all('p')
        description = nationSummary(nationsummary)

        nationsummarybox = soup.find('div', class_='nationsummarybox').find_all('td')
        box = summaryBox(nationsummarybox)

        overview = {
                    'flag': flag,
                    'short_name': short_name,
                    'long_name': long_name,
                    'wa_category': wa_category,
                    'motto': motto,
                    'bubbles': bubbles,
                    'description': description,
                    'box': box
                    }

        return overview

    def activity(self, filters: str):
        nation_name: str = self.nation_name
        formatted_name: str = format_text(nation_name)
        check_if_nation_exists(formatted_name)

        url: str = f'https://www.nationstates.net/page=activity/view=nation.{formatted_name}/filter={filters}'
        source: str = get_dynamic_source(url, '//*[@id="reports"]/ul')

        events = None

        if not source == None:        
            soup: bs = bs(source, DEFAULT_PARSER)
            reports = soup.find('div', class_='clickabletimes').find('ul')
            events = (li.get_text() for li in reports.find_all('li'))
            return events
        return events

    def census_generator(self, censusid: tuple | list) -> Iterator:
        nation_name: str = self.nation_name
        urls = census_url_generator(nation_name, (id for id in censusid))
        with ThreadPoolExecutor(max_workers=20) as executor:
            futures = {executor.submit(scrape_census, url): url for url in urls}
            for future in futures:
                yield future.result()

if __name__ == '__main__':
    data = N('orlys').census_generator([i for i in range(89)])

    for i in data:
        print(i)


from awesomeNations.exceptions import HTTPError
from awesomeNations.configuration import DEFAULT_HEADERS, DEFAULT_PARSER
from bs4 import BeautifulSoup as bs
import requests

def request(parser: str = DEFAULT_PARSER, url: str = 'https://www.nationstates.net/') -> bs:
    try:
        html = requests.get(url, headers=DEFAULT_HEADERS, timeout=(10, 5))
        if html.status_code != 200:
            raise HTTPError(html.status_code)
        response = bs(html.text, parser)
        return response
    except TimeoutError:
        raise HTTPError(html.status_code)

def format_text(text: str) -> str:
    formatted_text = text.lower().strip().replace(' ', '_')
    return formatted_text


from awesomeNations.exceptions import RegionNotFound
from awesomeNations.configuration import DEFAULT_PARSER
from awesomeNations.seleniumScrapper import get_dynamic_source
from awesomeNations.sharedMethods import request, format_text
from bs4 import BeautifulSoup as bs
from typing import Iterator

def check_if_region_exists(region_name: str) -> None:
    url: str = f'https://www.nationstates.net/region={region_name}'
    response: dict = request(parser='html.parser', url=url)
    soup: bs = response
    error_p = soup.find('p', class_="error")

    if error_p:
        raise RegionNotFound(region_name)

def embassy_generator(url: str, default_embassies_output: None) -> Iterator:
    source: str = get_dynamic_source(url, '//*[contains(concat( " ", @class, " " ), concat( " ", "divindent", " " ))] | //*[contains(concat( " ", @class, " " ), concat( " ", "mcollapse", " " ))]')
    try:
        soup: bs = bs(source, DEFAULT_PARSER)

        divindents: bs = soup.find_all('div', class_='divindent')
        embassy_table: bs = divindents[3].find('table', class_='shiny wide embassies mcollapse')

        for n, d in zip(embassy_table.find_all('td', class_='bigleft'), embassy_table.find_all('td', class_='')):
            name = n.get_text()
            duration = d.get_text()
            if str(name[0]).isnumeric():
                split: list = name.split(' ')
                split.pop(0)
                name = ' '.join(split)
            yield {'region': name, 'duration': duration}
    except:
        yield default_embassies_output

def scrape_world_census(url: str):
    try:
        response = request(parser='lxml', url=url)
        soup: bs = response

        region_rank: list = []

        rank_table = soup.find('table', class_='shiny ranks nationranks mcollapse').find_all('tr')
        rank_table.pop(0)

        rank_elements = [td.find_all('td', limit=2) for td in rank_table]
        for td in rank_elements:
            nation = td[1].get_text()
            region_rank.append(nation)
        
        page = soup.find('div', id='regioncontent')
        paragraphs = page.find_all('p', limit=2)

        description: str = paragraphs[0].get_text()
        region_world_rank: str = paragraphs[1].get_text()
        return {'title': page.find('h3').get_text(), 'description': description, 'region_world_rank': region_world_rank, 'rank': region_rank}
    except:
        return None

class R:
    def __init__(self, region_name: str):
        self.region_name = region_name
    
    def exists(self) -> bool:
        region_name: str = self.region_name
        url: str = f'https://www.nationstates.net/region={region_name}'
        response: bs = request(parser='lxml', url=url)
        error_p = response.find('p', class_="error")

        if error_p:
            return False
        else:
            return True

    def overview(self):
        region_name: str = self.region_name
        formatted_name: str = format_text(region_name)
        check_if_region_exists(formatted_name)

        url: str = f'https://www.nationstates.net/region={formatted_name}'
        response: dict = request(parser='lxml', url=url)
        soup: bs = response

        founder: str = None
        governor: str = None
        category: str = None
        wa_delegate: str = None
        last_wa_update: str = None
        region_flag: str = None
        region_banner: str = None

        region_cover = soup.find('div', class_='regioncover')

        flag_cover = region_cover.find('img')
        if flag_cover:
            if flag_cover.has_attr('src'):
                region_flag = f'https://www.nationstates.net{flag_cover.attrs['src']}'
            elif flag_cover.has_attr('data-cfsrc'):
                region_flag = f'https://www.nationstates.net{flag_cover.attrs['data-cfsrc']}'
        banner_source = region_cover.attrs['style'].replace('background-image:url', '').replace('(', '').replace(')', '')
        region_banner = f'https://www.nationstates.net{banner_source}'

        region_content = soup.find('div', id='regioncontent')
        paragraphs: list = region_content.find_all('p', limit=4)
        for text in paragraphs:
            content: str = text.get_text().strip()
            if 'Feeder' in content or 'Sinker' in content or 'Frontier' in content:
                category = content
            if 'Last WA Update' in content:
                last_wa_update = content
            if 'Governor' in content:
                governor = content
            if 'WA Delegate' in content:
                wa_delegate = content
            if 'Founder' in content:
                founder = content

        overview = dict(category=category, governor=governor, wa_delegate=wa_delegate, last_wa_update=last_wa_update, founder=founder, region_flag=region_flag, region_banner=region_banner)
        return overview

    def world_census(self, censusid: tuple | list) -> Iterator:
        region_name: str = self.region_name
        formatted_name: str = format_text(region_name)
        check_if_region_exists(formatted_name)

        for id in censusid:
            url: str = f'https://www.nationstates.net/page=list_nations/censusid={id}/region={formatted_name}'

            world_census: dict = scrape_world_census(url)
            yield world_census

    def activity(self, filter: str):
        region_name: str = self.region_name
        formatted_name: str = format_text(region_name)
        check_if_region_exists(formatted_name)

        url: str = f'https://www.nationstates.net/page=activity/view=region.{formatted_name}/filter={filter}'
        source: str = get_dynamic_source(url, '//*[@id="reports"]/ul')

        events = None

        if not source == None:        
            soup: bs = bs(source, DEFAULT_PARSER)
            reports = soup.find('div', class_='clickabletimes').find('ul')
            events = (li.get_text() for li in reports.find_all('li'))
            return events
        return events

    def embassies(self) -> Iterator:
        region_name: str = self.region_name
        formatted_name: str = format_text(region_name)
        check_if_region_exists(formatted_name)

        default_embassies_output: None = None
        url: str = f'https://www.nationstates.net/page=region_admin/region={formatted_name}'
        embassies: Iterator = embassy_generator(url, default_embassies_output)
        return embassies


from awesomeNations.nationMethods import N
from awesomeNations.regionMethods import R
from typing import Iterator
from datetime import datetime

class AwesomeNations:
    """
    Hello! I'm the main class of this library and I'm very excited to work with you!
    """
    def nationStates_birthday() -> bool:
        "Today is 11/13?"
        today = datetime.today()
        date: str = today.strftime('%D')
        birthday: bool = False
        if '11/13' in date:
            birthday = True
        return birthday

    def nationStates_age() -> str:
        "Current year - NationStates year of creation (NationStates was created in 2002)."
        created = 2002
        today = datetime.today().year
        age = today - created
        result = f'Around {age-1}-{age} years old.'
        return result

    class Nation:
        """
        Class dedicated to extracting data from NationStates nations.
        """
        def __init__(self, nation_name: str = 'testlandia') -> None:
            self.nation_name = nation_name

        def exists(self) -> bool:
            """
            Checks if nation exists.
            """
            exist = N(self.nation_name).exists()
            return exist
        
        def get_overview(self) -> dict:
            """
            Get an overview of the requested nation.
            """
            overview = N(self.nation_name).overview()
            return overview

        def get_activity(self, filters: str = 'all') -> Iterator:
            """
            Get national happenings of the requested nation.
            """
            activity = N.activity(self, filters)
            return activity

        def get_census(self, censusid: tuple | list = [0]) -> Iterator:
            """
            Gets one or more censuses [0-88] from the requested nation, examples:
            - [0]: Civil rights
            - [46]: Defense Forces
            - [0, 1, 2, 3] ... [88]
            """
            census = N(self.nation_name).census_generator(censusid)
            return census

    class Region:
        """
        Class dedicated to extracting data from NationStates regions.
        """
        def __init__(self, region_name: str = 'The Pacific') -> None:
            self.region_name = region_name
        
        def exists(self) -> bool:
            """
            Checks if the region exists.
            """
            exist = R.exists(self)
            return exist
        
        def get_overview(self) -> dict:
            """
            Provides an overview of the requested region.
            """
            overview = R.overview(self)
            return overview

        def get_world_census(self, censusid: tuple | list = [0]) -> Iterator:
            """
            Retrieves a world census of the requested region.
            """
            ranks = R.world_census(self, censusid)
            return ranks

        def get_activity(self, filters: str = 'all') -> Iterator:
            """
            Get regional happenings of the requested region.
            """
            activity = R.activity(self, filters)
            return activity

        def get_embassies(self) -> Iterator:
            """
            Retrieves embassies of the requested region.
            """
            embassies = R.embassies(self)
            return embassies

# Testing to see if my life is worth it:
if __name__ == '__main__':
    nation = (AwesomeNations.Nation('testlandia'))
    region = (AwesomeNations.Region('The Pacific'))

    print('NATION:')
    print(f'{nation.exists()=}\n')
    print(f'{nation.get_overview()=}\n')
    for activity in nation.get_activity():
        print(activity)
    print('\n')
    for census in nation.get_census([i for i in range(21)]):
        print(census)
    
    print('REGION:')
    print(f'{region.exists()=}\n')
    print(f'{region.get_overview()=}\n')
    for activity in region.get_activity():
        print(activity)
    for embassy in region.get_embassies():
        print(embassy)
    for rank in AwesomeNations.Region().get_world_census([i for i in range(21)]):
        print(rank, '\n')


from awesomeNations.configuration import DEFAULT_HEADERS

# Browser automation modules
from selenium import webdriver
from selenium.webdriver.firefox.service import Service
from selenium.webdriver.firefox.options import Options
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.common.exceptions import TimeoutException
from webdriver_manager.firefox import GeckoDriverManager 

def driver_setup():
    # Automatically installs the current browser version
    service = Service (GeckoDriverManager().install())

    options = Options()
    options.add_argument("--headless")
    options.add_argument(F"--user-agent={DEFAULT_HEADERS}")

    driver = webdriver.Chrome(service=service, options=options)
    return driver

# Wait until element x is present and then get the page source.
def get_dynamic_source(url: str, xpath: str):
    driver = driver_setup()
    driver.get(url)
    try:
        wait = WebDriverWait(driver, 5)
        driver.execute_script("window.scrollTo(0, document.body.scrollHeight);")
        element = wait.until(EC.presence_of_element_located((By.XPATH, xpath)))

        source = None
        if element.is_displayed():
            source = driver.page_source
        return source
    except:
        return None
    finally:
        driver.quit()

def script_runner(url: str, script_xpath: str):
    driver = driver_setup()
    driver.get(url)
    try:
        driver.execute_script(script_xpath)
        source = driver.page_source
        return source
    except TimeoutException:
        return None
    finally:
        driver.quit()

if __name__ == "__main__":
    get_dynamic_source('https://www.nationstates.net/page=region_admin/region=fullworthia', '//*[contains(concat( " ", @class, " " ), concat( " ", "divindent", " " ))] | //*[contains(concat( " ", @class, " " ), concat( " ", "mcollapse", " " ))]')
    print('get_dynamic_source completed')

    script_runner('https://www.nationstates.net/page=activity/view=nation.democratic_fun', '//*[@id="loggedout"]/script[13]')
    print('script_runner completed')


from typing import Final

DEFAULT_HEADERS: Final[str] = {'User-Agent': 'Mozilla/5.0 (iPhone; CPU iPhone OS 16_6 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/16.6 Mobile/15E148 Safari/604.1'}
DEFAULT_PARSER: Final[str] = 'html.parser'


from setuptools import setup, find_packages

def long_description() -> str:
    long_description: str = ''
    with open('README.md', 'r', encoding='utf-8') as file:
        long_description += file.read()
    with open('CHANGELOG.md', 'r', encoding='utf-8') as file:
        long_description += f'\n\n{file.read()}'
    return long_description

setup(
    name='awesomeNations',
    version='0.1.0',
    description='A simple python web scraping library for NationStates',
    long_description=long_description(),
    long_description_content_type='text/markdown',
    author='Orly Neto',
    author_email='orly2carvalhoneto@gmail.com',
    license='MIT License',
    keywords=['NationStates', 'Scrapper', 'Web Scrapper', 'NationStates scrapper'],
    packages=find_packages(),
    install_requires=['beautifulsoup4==4.12.3', 'requests==2.32.3', 'selenium==4.27.1', 'webdriver-manager==4.0.2', 'lxml==5.3.0'])


from random import random

def motivational_quotes():
    quotes = ['only put off until tomorrow what you are willing to die having left undone.',
              'all our dreams can come true, if we have the courage to pursue them.',
              'great things are done by a series of small things brought together.',
              "you've got to get up every morning with determination if you're going to go to bed with satisfaction.",
              'dream big. Work hard.',
              'you are your only limit.',
              "never be limited by other people's limited imaginations."
              'we cannot solve problems with the kind of thinking we employed when we came up with them.',
              'stay away from those people who try to disparage your ambitions. Small minds will always do that, but great minds will give you a feeling that you can become great too.',
              'Success is not final; failure is not fatal: It is the courage to continue that counts.',
              'it is better to fail in originality than to succeed in imitation.',
              'the road to success and the road to failure are almost exactly the same.',
              'develop success from failures. Discouragement and failure are two of the surest stepping stones to success.',
              'success is peace of mind, which is a direct result of self-satisfaction in knowing you made the effort to become the best of which you are capable.',
              'the pessimist sees difficulty in every opportunity. The optimist sees opportunity in every difficulty.',
              "you learn more from failure than from success. Don't let it stop you. Failure builds character.",
              'goal setting is the secret to a compelling future.',
              'setting goals is the first step in turning the invisible into the visible.',
              'your work is going to fill a large part of your life, and the only way to be truly satisfied is to do what you believe is great work. And the only way to do great work is to love what you do. If you haven’t found it yet, keep looking. Don’t settle. As with all matters of the heart, you’ll know when you find it.',
              'think like a queen. A queen is not afraid to fail. Failure is another stepping stone to greatness.',
              'take the attitude of a student, never be too big to ask questions, never know too much to learn something new.',
              'success is stumbling from failure to failure with no loss of enthusiasm.',
              'perfection is not attainable. But if we chase perfection we can catch excellence.',
              "get a good idea and stay with it. Dog it, and work at it until it's done right.",
              'optimism is the faith that leads to achievement. Nothing can be done without hope and confidence.'
              'work until your bank account looks like a phone number.',
              'talent wins games, but teamwork and intelligence win championships.',
              'teamwork is the ability to work together toward a common vision. The ability to direct individual accomplishments toward organizational objectives. It is the fuel that allows common people to attain uncommon results.',
              "don't let someone else's opinion of you become your reality.",
              'do the best you can. No one can do more than that.',
              'If you can dream it, you can do it.']
    index = int(len(quotes) * random())
    return quotes[index]

class NationNotFound(Exception):
    def __init__(self, nation_name) -> None:
        self.message = f'Nation "{nation_name}" not found, perhaps this nation no longer exists or never existed.'
        super().__init__(f'{self.message}')

class RegionNotFound(Exception):
    def __init__(self, region_name) -> None:
        self.message = f'Oh no, the region "{region_name}" does not exist. Maybe you forgot a comma? Or did you think programming was easy? Cute.\nJokes aside, {motivational_quotes()}'
        super().__init__(f'{self.message}')

class HTTPError(Exception):
    def __init__(self, status_code) -> None:
        self.message = f'HTTP error, status code: {status_code}. Hope This Totally Pleases-you!\nJokes aside, {motivational_quotes()}'
        super().__init__(f'{self.message}')